@@ -2,33 +2,37 @@
 # coding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 7)
+__version__ = (0, 0, 9)
 __all__ = ["urlopen", "request", "download"]
 
 import errno
 
 from collections.abc import Callable, Generator, Iterable, Mapping, Sequence
-from copy import copy
 from gzip import decompress as decompress_gzip
 from http.client import HTTPResponse
 from http.cookiejar import CookieJar
 from inspect import isgenerator
-from json import dumps, loads
 from os import fsdecode, fstat, makedirs, PathLike
 from os.path import abspath, dirname, isdir, join as joinpath
 from re import compile as re_compile
 from shutil import COPY_BUFSIZE # type: ignore
+from socket import getdefaulttimeout, setdefaulttimeout
 from ssl import SSLContext, _create_unverified_context
 from string import punctuation
+from types import EllipsisType
 from typing import cast, Any
 from urllib.error import HTTPError
 from urllib.parse import quote, urlencode, urlsplit
-from urllib.request import build_opener, HTTPCookieProcessor, HTTPSHandler, OpenerDirector, Request
+from urllib.request import (
+    build_opener, BaseHandler, HTTPCookieProcessor, HTTPSHandler, 
+    HTTPRedirectHandler, OpenerDirector, Request, 
+)
 from zlib import compressobj, DEF_MEM_LEVEL, DEFLATED, MAX_WBITS
 
 from argtools import argcount
 from filewrap import bio_skip_iter, SupportsWrite
 from http_response import get_filename, get_length, is_chunked, is_range_request
+from orjson import dumps, loads
 
 
 if "__del__" not in HTTPResponse.__dict__:
@@ -36,9 +40,18 @@ if "__del__" not in HTTPResponse.__dict__:
 if "__del__" not in OpenerDirector.__dict__:
     setattr(OpenerDirector, "__del__", OpenerDirector.close)
 
-_opener: None | OpenerDirector = None
+_opener: OpenerDirector = build_opener(HTTPSHandler(context=_create_unverified_context()))
 CRE_search_charset = re_compile(r"\bcharset=(?P<charset>[^ ;]+)").search
 
+if getdefaulttimeout() is None:
+    setdefaulttimeout(60)
+
+
+class NoRedirectHandler(HTTPRedirectHandler):
+
+    def redirect_request(self, req, fp, code, msg, headers, newurl):
+        return None
+
 
 def decompress_deflate(data: bytes, compresslevel: int = 9) -> bytes:
     # Fork from: https://stackoverflow.com/questions/1089662/python-inflate-and-deflate-implementations#answer-1089787
@@ -62,13 +75,20 @@ def decompress_deflate(data: bytes, compresslevel: int = 9) -> bytes:
     return deflated
 
 
+def get_charset(content_type: str, default="utf-8") -> str:
+    match = CRE_search_charset(content_type)
+    if match is None:
+        return "utf-8"
+    return match["charset"]
+
+
 def ensure_ascii_url(url: str, /) -> str:
     if url.isascii():
         return url
     return quote(url, safe=punctuation)
 
 
-def decompress_response(resp: HTTPResponse) -> bytes:
+def decompress_response(resp: HTTPResponse, /) -> bytes:
     data = resp.read()
     content_encoding = resp.headers.get("Content-Encoding")
     match content_encoding:
@@ -96,10 +116,10 @@ def urlopen(
     cookies: None | CookieJar = None, 
     proxy: None | tuple[str, str] = None, 
     context: None | SSLContext = None, 
+    allow_redirects: bool = True, 
     opener: None | OpenerDirector = None, 
     origin: None | str = None, 
 ) -> HTTPResponse:
-    global _opener
     if isinstance(url, str) and not urlsplit(url).scheme:
         if origin:
             if not url.startswith("/"):
@@ -113,11 +133,11 @@ def urlopen(
         if isinstance(json, bytes):
             data = json
         else:
-            data = dumps(json).encode("utf-8")
+            data = dumps(json)
         if headers:
-            headers = {**headers, "Content-type": "application/json"}
+            headers = {**headers, "Content-type": "application/json; charset=UTF-8"}
         else:
-            headers = {"Content-type": "application/json"}
+            headers = {"Content-type": "application/json; charset=UTF-8"}
     elif data is not None:
         if isinstance(data, bytes):
             pass
@@ -151,16 +171,19 @@ def urlopen(
     if proxy:
         req.set_proxy(*proxy)
     if opener is None:
-        if _opener is None:
-            opener = _opener = build_opener(HTTPSHandler(context=_create_unverified_context()))
+        handlers: list[BaseHandler] = []
+        if context is not None:
+            handlers.append(HTTPSHandler(context=context))
+        if cookies is not None:
+            handlers.append(HTTPCookieProcessor(cookies))
+        if not allow_redirects:
+            handlers.append(NoRedirectHandler())
+        if handlers:
+            if not isinstance(handlers[0], HTTPSHandler):
+                handlers.insert(0, HTTPSHandler(context=_create_unverified_context()))
+            opener = build_opener(*handlers)
         else:
             opener = _opener
-    if context is not None or cookies is not None:
-        opener = copy(opener)
-        if context is not None:
-            opener.add_handler(HTTPSHandler(context=context))
-        if cookies is not None:
-            opener.add_handler(HTTPCookieProcessor(cookies))
     req.full_url = ensure_ascii_url(req.full_url)
     if timeout is None:
         return opener.open(req)
@@ -168,19 +191,13 @@ def urlopen(
         return opener.open(req, timeout=timeout)
 
 
-def get_charset(content_type: str, default="utf-8") -> str:
-    match = CRE_search_charset(content_type)
-    if match is None:
-        return "utf-8"
-    return match["charset"]
-
-
 def request(
     url: str | Request, 
     method: str = "GET", 
-    parse: None | bool | Callable = None, 
+    parse: None | EllipsisType | bool | Callable = None, 
     raise_for_status: bool = True, 
     timeout: None | float = 60, 
+    allow_redirects: bool = True, 
     **request_kwargs, 
 ):
     request_kwargs.pop("stream", None)
@@ -189,6 +206,7 @@ def request(
             url=url, 
             method=method, 
             timeout=timeout, 
+            allow_redirects=allow_redirects, 
             **request_kwargs, 
         )
     except HTTPError as e:
@@ -197,6 +215,9 @@ def request(
         resp = getattr(e, "file")
     if parse is None:
         return resp
+    elif parse is ...:
+        resp.close()
+        return resp
     with resp:
         if isinstance(parse, bool):
             data = decompress_response(resp)
